@@ -1,12 +1,12 @@
-# 
-# Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
+#
+# Copyright (c) 1994-2008 Carnegie Mellon University.  All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
 # are met:
 #
 # 1. Redistributions of source code must retain the above copyright
-#    notice, this list of conditions and the following disclaimer. 
+#    notice, this list of conditions and the following disclaimer.
 #
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in
@@ -15,14 +15,15 @@
 #
 # 3. The name "Carnegie Mellon University" must not be used to
 #    endorse or promote products derived from this software without
-#    prior written permission. For permission or any other legal
-#    details, please contact  
-#      Office of Technology Transfer
+#    prior written permission. For permission or any legal
+#    details, please contact
 #      Carnegie Mellon University
-#      5000 Forbes Avenue
-#      Pittsburgh, PA  15213-3890
-#      (412) 268-4387, fax: (412) 268-7395
-#      tech-transfer@andrew.cmu.edu
+#      Center for Technology Transfer and Enterprise Creation
+#      4615 Forbes Avenue
+#      Suite 302
+#      Pittsburgh, PA  15213
+#      (412) 268-7393, fax: (412) 268-7395
+#      innovation@andrew.cmu.edu
 #
 # 4. Redistributions of any form whatsoever must retain the following
 #    acknowledgment:
@@ -37,7 +38,7 @@
 # AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 # OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #
-# $Id: Shell.pm,v 1.35 2004/01/15 14:35:35 ken3 Exp $
+# $Id: Shell.pm,v 1.44 2010/01/06 17:01:55 murch Exp $
 #
 # A shell framework for Cyrus::IMAP::Admin
 #
@@ -126,7 +127,7 @@ my %builtins = (exit =>
 		  [\&_sc_info, '[mailbox]',
 		   'display mailbox/server metadata'],
 		mboxcfg =>
-		  [\&_sc_mboxcfg, 'mailbox [comment|news2mail|expire|squat] value',
+		  [\&_sc_mboxcfg, 'mailbox [comment|condstore|expire|news2mail|sieve|squat|/<explicit annotation>] value',
 		   'configure mailbox'],
 		mboxconfig => 'mboxcfg',
 		reconstruct =>
@@ -159,6 +160,14 @@ my %builtins = (exit =>
 		   '[--partition partition] mailbox server [partition]',
 		   'transfer (relocate) a mailbox to a different server'],
 		xfer => 'xfermailbox',
+		subscribe =>
+                  [\&_sc_subscribe, '[mailbox]',
+                    'subscribe to a mailbox'],
+                sub => 'subscribe',
+		unsubscribe =>
+		  [\&_sc_unsubscribe, '[mailbox]',
+		     'unsubscribe from a mailbox'],
+		unsub => 'unsubscribe',
 		#? alias
 		#? unalias
 		#? load
@@ -428,9 +437,10 @@ sub run {
 # (It's not as trivial as run() because it does things expected of standalone
 # programs, as opposed to things expected from within a program.)
 sub shell {
-  my ($server, $port, $authz, $auth, $systemrc, $userrc, $dorc, $mech, $pw) =
+  my ($server, $port, $authz, $auth, $systemrc, $userrc, $dorc, $mech, $pw,
+      $tlskey, $notls) =
     ('', 143, undef, $ENV{USER} || $ENV{LOGNAME}, '/usr/local/etc/cyradmrc.pl',
-     "$ENV{HOME}/.cyradmrc.pl", 1, undef, undef);
+     "$ENV{HOME}/.cyradmrc.pl", 1, undef, undef, undef, undef);
   GetOptions('user|u=s' => \$auth,
 	     'authz|z=s' => \$authz,
 	     'rc|r!' => \$dorc,
@@ -440,6 +450,8 @@ sub shell {
 	     'port|p=i' => \$port,
 	     'auth|a=s' => \$mech,
 	     'password|w=s' => \$pw,
+  	     'tlskey|t:s' => \$tlskey,
+  	     'notls' => \$notls,
 	     'help|h' => sub { cyradm_usage(); exit(0); }
 	    );
   if ($server ne '' && @ARGV) {
@@ -458,7 +470,8 @@ sub shell {
 			  -callback => \&_cb_eof,
 			  -rock => \$cyradm});
     $cyradm->authenticate(-authz => $authz, -user => $auth,
-			  -mechanism => $mech, -password => $pw)
+			  -mechanism => $mech, -password => $pw,
+			  -tlskey => $tlskey, -notls => $notls)
       or die "cyradm: cannot authenticate to server with $mech as $auth\n";
   }
   my $fstk = [*STDIN, *STDOUT, *STDERR];
@@ -686,13 +699,25 @@ sub _sc_listacl {
       my $flags = $mbx->[1];
       next if($flags =~ /(\\noselect|\\nonexistent|\\placeholder)/i);
       $lfh->[1]->print($name,":\n");
-      $nargv[0] = $name;
-      if(showacl(2,@nargv) != 0) {
-	return 1;
+      my %acl = $$cyrref->listaclmailbox($name);
+      if (defined $$cyrref->error) {
+         $lfh->[2]->print($$cyrref->error, "\n");
+         next;
+      }
+      foreach my $acl (keys %acl) {
+        $lfh->[1]->print("  ", $acl, " ", $acl{$acl}, "\n");
       }
     }
   } else {
-    return showacl(0,@nargv);
+    my %acl = $$cyrref->listaclmailbox(@nargv);
+    if (defined $$cyrref->error) {
+       $lfh->[2]->print($$cyrref->error, "\n");
+       return 1;
+    }
+
+    foreach my $acl (keys %acl) {
+      $lfh->[1]->print($acl, " ", $acl{$acl}, "\n");
+    }
   }
   return 0;
 }
@@ -765,6 +790,20 @@ sub _sc_auth {
       $want = '-service';
       next;
     }
+    if (Cyrus::IMAP::havetls()) {
+      if ($opt ne '' && '-tlskey' =~ /^\Q$opt/ || $opt eq '--tlskey') {
+	$want = '-tlskey';
+	next;
+      }
+      if ($opt ne '' && '-notls' =~ /^\Q$opt/ || $opt eq '--notls') {
+	$want = '-notls';
+	next;
+      }
+      if ($opt =~ /^-/) {
+	die "usage: authenticate [-minssf N] [-maxssf N] [-mechanisms STR]\n".
+	    "                    [-service name] [-tlskey keyfile] [-notls] [user]\n";
+      }
+    }
     if ($opt =~ /^-/) {
       die "usage: authenticate [-minssf N] [-maxssf N] [-mechanisms STR]\n".
 	  "                    [-service name] [user]\n";
@@ -776,8 +815,13 @@ sub _sc_auth {
   }
   push(@nargv, @argv);
   if (@nargv > 1) {
-    die "usage: authenticate [-minssf N] [-maxssf N] [-mechanisms STR]\n".
-        "                    [-service name] [user]\n";
+    if (Cyrus::IMAP::havetls()) {
+      die "usage: authenticate [-minssf N] [-maxssf N] [-mechanisms STR]\n".
+          "                    [-service name] [-tlskey keyfile] [-notls] [user]\n";
+    } else {
+      die "usage: authenticate [-minssf N] [-maxssf N] [-mechanisms STR]\n".
+          "                    [-service name] [user]\n";
+    }
   }
   if (@nargv) {
     $opts{-user} = shift(@nargv);
@@ -1332,6 +1376,60 @@ sub _sc_info {
   0;
 }
 
+sub _sc_subscribe {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    # gack.  bloody tcl.
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: subscribe [mailbox]\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (!$cyrref || !$$cyrref) {
+    die "subscribe: no connection to server\n";
+  }
+  $$cyrref->subscribe(@nargv);
+  if (defined $$cyrref->error) {
+    $lfh->[2]->print($$cyrref->error, "\n");
+    return 1;
+  }
+  0;
+}
+
+sub _sc_unsubscribe {
+  my ($cyrref, $name, $fh, $lfh, @argv) = @_;
+  my (@nargv, $opt);
+  shift(@argv);
+  while (defined ($opt = shift(@argv))) {
+    # gack.  bloody tcl.
+    last if $opt eq '--';
+    if ($opt =~ /^-/) {
+      die "usage: unsubscribe [mailbox]\n";
+    }
+    else {
+      push(@nargv, $opt);
+      last;
+    }
+  }
+  push(@nargv, @argv);
+  if (!$cyrref || !$$cyrref) {
+    die "unsubscribe: no connection to server\n";
+  }
+  $$cyrref->unsubscribe(@nargv);
+  if (defined $$cyrref->error) {
+    $lfh->[2]->print($$cyrref->error, "\n");
+    return 1;
+  }
+  0;
+}
+
 sub _sc_mboxcfg {
   my ($cyrref, $name, $fh, $lfh, @argv) = @_;
   my (@nargv, $opt);
@@ -1339,7 +1437,7 @@ sub _sc_mboxcfg {
   while (defined ($opt = shift(@argv))) {
     last if $opt eq '--';
     if ($opt =~ /^-/) {
-      die "usage: mboxconfig mailbox [comment|news2mail|expire|squat] value\n";
+      die "usage: mboxconfig mailbox [comment|condstore|expire|news2mail|sharedseen|sieve|squat|/<explicit annotation>] value\n";
     }
     else {
       push(@nargv, $opt);
@@ -1348,7 +1446,7 @@ sub _sc_mboxcfg {
   }
   push(@nargv, @argv);
   if (@nargv < 2) {
-    die "usage: mboxconfig mailbox [comment|news2mail|expire|squat] value\n";
+    die "usage: mboxconfig mailbox [comment|condstore|expire|news2mail|sharedseen|sieve|squat|/<explicit annotation>] value\n";
   }
   if (!$cyrref || !$$cyrref) {
     die "mboxconfig: no connection to server\n";
@@ -1492,9 +1590,8 @@ Remove ACLs from the specified mailbox.
 Delete the specified mailbox.
 
 Administrators do not have implicit delete rights on mailboxes.  Use the
-B<setaclmailbox> command to grant the C<c> permission (or other permission
-as specified by the deleteright configuration option in imapd.conf)
-to your principal if you need to delete a mailbox you do not own. 
+B<setaclmailbox> command to grant the C<k> permission to your
+principal if you need to delete a mailbox you do not own.
 
 Note that the online help admits to an optional host argument.  This argument
 is not currently used, and will be rejected with an error if specified; it
@@ -1579,19 +1676,35 @@ The currently supported attributes are:
 
 Sets a comment or description associated with the mailbox.
 
+=item C<condstore>
+
+Enables the IMAP CONDSTORE extension (modification sequences) on the mailbox.
+
 =item C<expire>
 
 Sets the number of days after which messages will be expired from the mailbox.
 
-=item C<squat>
-
-Indicates that the mailbox should have a squat index created for it.
-
 =item C<news2mail>
 
 Sets an email address to which messages injected into the server via NNTP 
 will be sent.
 
+=item C<sharedseen>
+
+Enables the use of a shared \Seen flag on messages rather than a
+per-user \Seen flag.  The 's' right in the mailbox ACL still controls
+whether a user can set the shared \Seen flag.
+
+=item C<sieve>
+
+Indicates the name of the global sieve script that should be run when
+a message is delivered to the shared mailbox (not used for personal
+mailboxes).
+
+=item C<squat>
+
+Indicates that the mailbox should have a squat index created for it.
+
 =back 
 
 =item C<renamemailbox> [C<--partition> I<partition>] I<oldname> I<newname>
@@ -1633,21 +1746,23 @@ of the connected server.
 
 Set ACLs on a mailbox.  The ACL may be one of the special strings C<none>,
 C<read> (C<lrs>), C<post> (C<lrsp>), C<append> (C<lrsip>), C<write>
-(C<lrswipcd>), or C<all> (C<lrswipcda>), or any combinations of the ACL codes:
+(C<lrswipkxte>), C<delete> (C<lrxte>), or C<all> (C<lrswipkxte>), or
+any combinations of the ACL codes:
 
 =over 4
 
 =item l
 
-Lookup (visible to LIST/LSUB/UNSEEN)
+Lookup (mailbox is visible to LIST/LSUB, SUBSCRIBE mailbox)
 
 =item r
 
-Read (SELECT, CHECK, FETCH, PARTIAL, SEARCH, COPY source)
+Read (SELECT/EXAMINE the mailbox, perform STATUS)
 
 =item s
 
-Seen (STORE \SEEN)
+Seen (set/clear \SEEN flag via STORE, also set \SEEN flag during
+    APPEND/COPY/FETCH BODY[...])
 
 =item w
 
@@ -1661,17 +1776,26 @@ Insert (APPEND, COPY destination)
 
 Post (send mail to mailbox)
 
-=item c
+=item k
 
-Create (subfolders)
+Create mailbox (CREATE new sub-mailboxes, parent for new mailbox in RENAME)
 
-=item d
+=item x
 
-Delete (STORE \DELETED, EXPUNGE)
+Delete mailbox (DELETE mailbox, old mailbox name in RENAME)
+
+=item t
+
+Delete messages (set/clear \DELETED flag via STORE, also set \DELETED
+    flag during APPEND/COPY)
+
+=item e
+
+Perform EXPUNGE and expunge as part of CLOSE
 
 =item a
 
-Administer (SETACL)
+Administer (SETACL/DELETEACL/GETACL/LISTRIGHTS)
 
 =back
 
@@ -1684,8 +1808,8 @@ The currently supported attributes are:
 
 =item C<motd>
 
-Sets a "message of the day".  The message gets displayed as an ALERT after
-authentication.
+Sets a "message of the day".  The message gets displayed as an ALERT upon
+connection.
 
 =item C<comment>
 
